/* $Id: lifedraw.c,v 1.2 2004/01/18 09:54:14 muli Exp $ */ 

#include <stdio.h> 
#include <unistd.h> 
#include <string.h> 
#include <curses.h> 
#include <signal.h>
#include <stdlib.h> 
#include <unistd.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static int gkfd; 

static void finish(int kfd, int status); 

static int kfd_setup(const int starting[][2], size_t starting_size)
{
	int kfd; 
	size_t sz = starting_size; 

	if ((kfd = open("/dev/klife", O_RDWR)) < 0) { 
		perror("open /dev/klife"); 
		exit(EXIT_FAILURE); 
	} 
	
	if (write(kfd, starting, sz) != sz) { 
		perror("write"); 
		close(kfd); 
		exit(EXIT_FAILURE); 
	} 
	
	gkfd = kfd; /* sighandler... */ 
	return kfd; 
}

static size_t generation(int kfd, char* buffer, size_t sz)
{
	int ret; 

	if ((ret = read(kfd, buffer, sz)) < 0) { 
		perror("read"); 
		finish(kfd, EXIT_FAILURE); 
	} 
	if (ret == 0) { 
		fprintf(stderr, "read returned EOF?!\n"); 
		finish(kfd, EXIT_FAILURE); 
	}

	return ret; 
}

static void draw(const char* buffer, size_t len)
{
	mvaddstr(0, 0, buffer); 

	refresh(); 
}

static void finish_sighand(int sig)
{
	finish(gkfd, EXIT_SUCCESS); 
}

static void finish(int kfd, int status)
{
	endwin();
	close(kfd); 
	exit(status);
}

static void curses_init(void)
{
	/* initialize your non-curses data structures here */
	(void)signal(SIGINT, finish_sighand);      /* arrange interrupts to terminate */

	(void)initscr();      /* initialize the curses library */
	keypad(stdscr, TRUE);  /* enable keyboard mapping */
	(void)nonl();         /* tell curses not to do NL->CR/NL on output */
	(void)cbreak();       /* take input chars one at a time, no wait for \n */
	(void)noecho();       /* don't echo input */
}

int main(void)
{
	int kfd; 
	const int rpentomino[][2] = {{13, 14} , {13, 15}, {14, 13}, {14, 14}, {15, 14}, {-1, -1}}; 
#if 0 
	const int glider[][2] = {{13, 14}, {14, 14}, {14, 15}, {15, 13}, {15, 15}, {-1, -1}}; 
#endif /* 0 */ 
#define starting rpentomino

	char buf[4096]; 
	size_t sz; 

	kfd = kfd_setup(starting, sizeof(starting)); 

	curses_init(); 

	while (1) { 
		sz = generation(kfd, buf, sizeof(buf)); 
		draw(buf, sz); 
		sleep(1); 
	} 

	finish(kfd, EXIT_SUCCESS); 
}

